@@ -14,19 +14,19 @@ DbConnection::~DbConnection() {
   LOG_VERBOSE;
 
   if (is_valid()) {
-    warning("call dbDisconnect() when finished working with a connection");
+    cpp11::warning(std::string("call dbDisconnect() when finished working with a connection"));
     disconnect();
   }
 }
 
-void DbConnection::connect(const Nullable<std::string>& host, const Nullable<std::string>& user,
-                           const Nullable<std::string>& password, const Nullable<std::string>& db,
-                           unsigned int port, const Nullable<std::string>& unix_socket,
-                           unsigned long client_flag, const Nullable<std::string>& groups,
-                           const Nullable<std::string>& default_file,
-                           const Nullable<std::string>& ssl_key, const Nullable<std::string>& ssl_cert,
-                           const Nullable<std::string>& ssl_ca, const Nullable<std::string>& ssl_capath,
-                           const Nullable<std::string>& ssl_cipher,
+void DbConnection::connect(const cpp11::sexp& host, const cpp11::sexp& user,
+                           const cpp11::sexp& password, const cpp11::sexp& db,
+                           unsigned int port, const cpp11::sexp& unix_socket,
+                           unsigned long client_flag, const cpp11::sexp& groups,
+                           const cpp11::sexp& default_file,
+                           const cpp11::sexp& ssl_key, const cpp11::sexp& ssl_cert,
+                           const cpp11::sexp& ssl_ca, const cpp11::sexp& ssl_capath,
+                           const cpp11::sexp& ssl_cipher,
                            int timeout, bool reconnect) {
   LOG_VERBOSE;
 
@@ -36,22 +36,22 @@ void DbConnection::connect(const Nullable<std::string>& host, const Nullable<std
   mysql_options(this->pConn_, MYSQL_OPT_LOCAL_INFILE, &local_infile);
   // Default to UTF-8
   mysql_options(this->pConn_, MYSQL_SET_CHARSET_NAME, "utf8mb4");
-  if (!groups.isNull())
+  if (Rf_isNull(groups))
     mysql_options(this->pConn_, MYSQL_READ_DEFAULT_GROUP,
-                  as<std::string>(groups).c_str());
-  if (!default_file.isNull())
+                  cpp11::as_cpp<std::string>(groups).c_str());
+  if (!Rf_isNull(default_file))
     mysql_options(this->pConn_, MYSQL_READ_DEFAULT_FILE,
-                  as<std::string>(default_file).c_str());
+                  cpp11::as_cpp<std::string>(default_file).c_str());
 
-  if (!ssl_key.isNull() || !ssl_cert.isNull() || !ssl_ca.isNull() ||
-      !ssl_capath.isNull() || !ssl_cipher.isNull()) {
+  if (!Rf_isNull(ssl_key) || !Rf_isNull(ssl_cert) || !Rf_isNull(ssl_ca) ||
+      !Rf_isNull(ssl_capath) || !Rf_isNull(ssl_cipher)) {
     mysql_ssl_set(
       this->pConn_,
-      ssl_key.isNull() ? NULL : as<std::string>(ssl_key).c_str(),
-      ssl_cert.isNull() ? NULL : as<std::string>(ssl_cert).c_str(),
-      ssl_ca.isNull() ? NULL : as<std::string>(ssl_ca).c_str(),
-      ssl_capath.isNull() ? NULL : as<std::string>(ssl_capath).c_str(),
-      ssl_cipher.isNull() ? NULL : as<std::string>(ssl_cipher).c_str()
+      Rf_isNull(ssl_key) ? NULL : cpp11::as_cpp<std::string>(ssl_key).c_str(),
+      Rf_isNull(ssl_cert) ? NULL : cpp11::as_cpp<std::string>(ssl_cert).c_str(),
+      Rf_isNull(ssl_ca) ? NULL : cpp11::as_cpp<std::string>(ssl_ca).c_str(),
+      Rf_isNull(ssl_capath) ? NULL : cpp11::as_cpp<std::string>(ssl_capath).c_str(),
+      Rf_isNull(ssl_cipher) ? NULL : cpp11::as_cpp<std::string>(ssl_cipher).c_str()
     );
   }
   if (timeout > 0) {
@@ -66,18 +66,18 @@ void DbConnection::connect(const Nullable<std::string>& host, const Nullable<std
   LOG_VERBOSE;
 
   if (!mysql_real_connect(this->pConn_,
-                          host.isNull() ? NULL : as<std::string>(host).c_str(),
-                          user.isNull() ? NULL : as<std::string>(user).c_str(),
-                          password.isNull() ? NULL : as<std::string>(password).c_str(),
-                          db.isNull() ? NULL : as<std::string>(db).c_str(),
+                          Rf_isNull(host) ? NULL : cpp11::as_cpp<std::string>(host).c_str(),
+                          Rf_isNull(user) ? NULL : cpp11::as_cpp<std::string>(user).c_str(),
+                          Rf_isNull(password) ? NULL : cpp11::as_cpp<std::string>(password).c_str(),
+                          Rf_isNull(db) ? NULL : cpp11::as_cpp<std::string>(db).c_str(),
                           port,
-                          unix_socket.isNull() ? NULL : as<std::string>(unix_socket).c_str(),
+                          Rf_isNull(unix_socket) ? NULL : cpp11::as_cpp<std::string>(unix_socket).c_str(),
                           client_flag)) {
     std::string error = mysql_error(this->pConn_);
     mysql_close(this->pConn_);
     this->pConn_ = NULL;
 
-    stop("Failed to connect: %s", error.c_str());
+    cpp11::stop("Failed to connect: %s", error.c_str());
   }
 }
 
@@ -85,11 +85,8 @@ void DbConnection::disconnect() {
   if (!is_valid()) return;
 
   if (has_query()) {
-    warning(
-      "%s\n%s",
-      "There is a result object still in use.",
-      "The connection will be automatically released when it is closed"
-    );
+    cpp11::warning(std::string("There is a result object still in use.\n"
+      "The connection will be automatically released when it is closed"));
   }
 
   try {
@@ -105,40 +102,42 @@ bool DbConnection::is_valid() {
 
 void DbConnection::check_connection() {
   if (!is_valid()) {
-    stop("Invalid or closed connection");
+    cpp11::stop("Invalid or closed connection");
   }
 }
 
-List DbConnection::info() {
+cpp11::list DbConnection::info() {
+  using namespace cpp11::literals;
   return
-    List::create(
-      _["host"] = std::string(pConn_->host),
-      _["username"] = std::string(pConn_->user),
-      _["dbname"] = std::string(pConn_->db ? pConn_->db : ""),
-      _["con.type"] = std::string(mysql_get_host_info(pConn_)),
-      _["db.version"] = std::string(mysql_get_server_info(pConn_)),
-      _["port"] = NA_INTEGER,
-      _["protocol.version"] = (int) mysql_get_proto_info(pConn_),
-      _["thread.id"] = (int) mysql_thread_id(pConn_)
-    );
+    cpp11::list({
+      "host"_nm = std::string(pConn_->host),
+      "username"_nm = std::string(pConn_->user),
+      "dbname"_nm = std::string(pConn_->db ? pConn_->db : ""),
+      "con.type"_nm = std::string(mysql_get_host_info(pConn_)),
+      "db.version"_nm = std::string(mysql_get_server_info(pConn_)),
+      "db.version.int"_nm = (int) mysql_get_server_version(pConn_),
+      "port"_nm = NA_INTEGER,
+      "protocol.version"_nm = (int) mysql_get_proto_info(pConn_),
+      "thread.id"_nm = (int) mysql_thread_id(pConn_)
+    });
 }
 
 MYSQL* DbConnection::get_conn() {
   return pConn_;
 }
 
-SEXP DbConnection::quote_string(const String& input) {
+SEXP DbConnection::quote_string(const cpp11::r_string& input) {
   if (input == NA_STRING)
     return get_null_string();
 
-  const char* input_cstr = input.get_cstring();
-  size_t input_len = strlen(input_cstr);
+  const auto input_str = static_cast<std::string>(input);
+  const auto input_len = input_str.size();
 
   // Create buffer with enough room to escape every character
   std::string output = "'";
   output.resize(input_len * 2 + 3);
 
-  size_t end = mysql_real_escape_string(pConn_, &output[1], input_cstr, input_len);
+  size_t end = mysql_real_escape_string(pConn_, &output[1], input_str.c_str(), input_len);
 
   output.resize(end + 1);
   output.append("'");
@@ -146,7 +145,7 @@ SEXP DbConnection::quote_string(const String& input) {
 }
 
 SEXP DbConnection::get_null_string() {
-  static RObject null = Rf_mkCharCE("NULL", CE_UTF8);
+  static auto null = Rf_mkCharCE("NULL", CE_UTF8);
   return null;
 }
 
@@ -156,7 +155,7 @@ void DbConnection::set_current_result(DbResult* pResult) {
 
   if (pCurrentResult_ != NULL) {
     if (pResult != NULL)
-      warning("Cancelling previous query");
+      cpp11::warning("Cancelling previous query");
 
     pCurrentResult_->close();
   }
@@ -184,7 +183,7 @@ bool DbConnection::exec(const std::string& sql) {
   check_connection();
 
   if (mysql_real_query(pConn_, sql.data(), sql.size()) != 0)
-    stop("Error executing query: %s", mysql_error(pConn_));
+    cpp11::stop("Error executing query: %s", mysql_error(pConn_));
 
   MYSQL_RES* res = mysql_store_result(pConn_);
   if (res != NULL)
@@ -196,14 +195,14 @@ bool DbConnection::exec(const std::string& sql) {
 }
 
 void DbConnection::begin_transaction() {
-  if (is_transacting()) stop("Nested transactions not supported.");
+  if (is_transacting()) cpp11::stop("Nested transactions not supported.");
   check_connection();
 
   transacting_ = true;
 }
 
 void DbConnection::commit() {
-  if (!is_transacting()) stop("Call dbBegin() to start a transaction.");
+  if (!is_transacting()) cpp11::stop("Call dbBegin() to start a transaction.");
   check_connection();
 
   mysql_commit(get_conn());
@@ -211,7 +210,7 @@ void DbConnection::commit() {
 }
 
 void DbConnection::rollback() {
-  if (!is_transacting()) stop("Call dbBegin() to start a transaction.");
+  if (!is_transacting()) cpp11::stop("Call dbBegin() to start a transaction.");
   check_connection();
 
   mysql_rollback(get_conn());
