@@ -10,11 +10,11 @@ public:
   virtual void send_query(const std::string& sql) = 0;
   virtual void close() = 0;
 
-  virtual void bind(const List& params) = 0;
+  virtual void bind(const cpp11::list& params) = 0;
 
-  virtual List get_column_info() = 0;
+  virtual cpp11::writable::list get_column_info() = 0;
 
-  virtual List fetch(int n_max = -1) = 0;
+  virtual cpp11::list fetch(int n_max = -1) = 0;
 
   virtual int n_rows_affected() = 0;
   virtual int n_rows_fetched() = 0;
