@@ -1,39 +1,39 @@
 #include "pch.h"
 #include "MariaTypes.h"
 
-List df_resize(const List& df, int n) {
-  R_xlen_t p = df.size();
+cpp11::list df_resize(const cpp11::list& df, int n) {
+  const auto p = df.size();
 
-  List out(p);
+  cpp11::writable::list out(p);
   for (R_xlen_t j = 0; j < p; ++j) {
     out[j] = Rf_lengthgets(df[j], n);
   }
 
   out.attr("names") = df.attr("names");
   out.attr("class") = df.attr("class");
-  out.attr("row.names") = IntegerVector::create(NA_INTEGER, -n);
+  out.attr("row.names") = cpp11::integers({NA_INTEGER, -n});
 
   return out;
 }
 
-void df_s3(const List& df, const std::vector<MariaFieldType>& types) {
+void df_s3(const cpp11::list& df, const std::vector<MariaFieldType>& types) {
   R_xlen_t p = df.size();
 
   for (R_xlen_t j = 0; j < p; ++j) {
-    RObject col(df[j]);
+    cpp11::sexp col(df[j]);
     switch (types[j]) {
     case MY_DATE:
-      col.attr("class") = CharacterVector::create("Date");
+      col.attr("class") = cpp11::strings(cpp11::as_sexp(cpp11::r_string("Date")));
       break;
     case MY_DATE_TIME:
-      col.attr("class") = CharacterVector::create("POSIXct", "POSIXt");
+      col.attr("class") = cpp11::strings({"POSIXct", "POSIXt"});
       break;
     case MY_TIME:
-      col.attr("class") = CharacterVector::create("hms", "difftime");
+      col.attr("class") = cpp11::strings({"hms", "difftime"});
       col.attr("units") = "secs";
       break;
     case MY_INT64:
-      col.attr("class") = CharacterVector::create("integer64");
+      col.attr("class") = cpp11::strings(cpp11::as_sexp(cpp11::r_string("integer64")));
       break;
     default:
       break;
@@ -42,17 +42,19 @@ void df_s3(const List& df, const std::vector<MariaFieldType>& types) {
   }
 }
 
-List df_create(const std::vector<MariaFieldType>& types, const std::vector<std::string>& names, int n) {
+cpp11::writable::list df_create(const std::vector<MariaFieldType>& types, const std::vector<std::string>& names, int n) {
   R_xlen_t p = types.size();
 
-  List out(p);
-  StringVector names_utf8 = wrap(names);
+  cpp11::writable::list out(p);
+  auto names_utf8 = static_cast<cpp11::writable::strings>(cpp11::as_sexp(names));
   for (int j = 0; j < names_utf8.size(); ++j) {
-    names_utf8[j] = Rf_mkCharCE(names_utf8[j], CE_UTF8);
+    const cpp11::r_string name = names_utf8[j];
+    const auto name_str = static_cast<std::string>(name);
+    names_utf8[j] = Rf_mkCharCE(name_str.c_str(), CE_UTF8);
   }
   out.attr("names") = names_utf8;
   out.attr("class") = "data.frame";
-  out.attr("row.names") = IntegerVector::create(NA_INTEGER, -n);
+  out.attr("row.names") = cpp11::integers({NA_INTEGER, -n});
 
   for (R_xlen_t j = 0; j < p; ++j) {
     out[j] = Rf_allocVector(type_sexp(types[j]), n);
